"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mergeConfigs = mergeConfigs;

function mergePluginsField(defaultPlugins = [], customPlugins = []) {
  return [...defaultPlugins, ...customPlugins];
}

function mergeRulesField(defaultRules = [], customRules = []) {
  return [...defaultRules, ...customRules];
}

function mergeExtensionsField({
  extensions: defaultExtensions = []
}, {
  extensions: customExtensions = []
}) {
  return [...defaultExtensions, ...customExtensions];
}

function mergeAliasField({
  alias: defaultAlias = {}
}, {
  alias: customAlias = {}
}) {
  return Object.assign({}, defaultAlias, customAlias);
}

function mergeModuleField(a, b) {
  return Object.assign({}, a, b, {
    rules: mergeRulesField(a.rules || [], b.rules || [])
  });
}

function mergeResolveField({
  resolve: defaultResolve = {}
}, {
  resolve: customResolve = {}
}) {
  return Object.assign({}, defaultResolve, customResolve, {
    alias: mergeAliasField(defaultResolve, customResolve),
    extensions: mergeExtensionsField(defaultResolve, customResolve)
  });
}

function mergeOptimizationField({
  optimization: defaultOptimization = {}
}, {
  optimization: customOptimization = {}
}) {
  return Object.assign({}, defaultOptimization, customOptimization);
}

function mergeConfigs(config, customConfig) {
  return Object.assign({}, customConfig, config, {
    devtool: customConfig.devtool || config.devtool,
    plugins: mergePluginsField(config.plugins, customConfig.plugins),
    module: mergeModuleField(config.module || {}, customConfig.module || {}),
    resolve: mergeResolveField(config, customConfig),
    optimization: mergeOptimizationField(config, customConfig)
  });
}