"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toImportFn = toImportFn;
exports.toImportFnPart = toImportFnPart;
exports.webpackIncludeRegexp = webpackIncludeRegexp;

var _tsDedent = require("ts-dedent");

var _coreCommon = require("@storybook/core-common");

var _importPipeline = require("./importPipeline");

function webpackIncludeRegexp(specifier) {
  const {
    directory,
    files
  } = specifier; // It appears webpack passes *something* similar to the absolute path to the file
  // on disk (prefixed with something unknown) to the matcher.
  // We don't want to include the absolute path in our bundle, so we will just pull any leading
  // `./` or `../` off our directory and match on that.
  // It's imperfect as it could match extra things in extremely unusual cases, but it'll do for now.
  // NOTE: directory is "slashed" so will contain only `/` (no `\`), even on windows

  const directoryWithoutLeadingDots = directory.replace(/^(\.+\/)+/, '/');
  const webpackIncludeGlob = ['.', '..'].includes(directory) ? files : `${directoryWithoutLeadingDots}/${files}`;
  const webpackIncludeRegexpWithCaret = (0, _coreCommon.globToRegexp)(webpackIncludeGlob); // picomatch is creating an exact match, but we are only matching the end of the filename

  return new RegExp(webpackIncludeRegexpWithCaret.source.replace(/^\^/, ''));
}

function toImportFnPart(specifier) {
  const {
    directory,
    importPathMatcher
  } = specifier;
  return (0, _tsDedent.dedent)`
      async (path) => {
        if (!${importPathMatcher}.exec(path)) {
          return;
        }

        const pathRemainder = path.substring(${directory.length + 1});
        return import(
          /* webpackChunkName: "[request]" */
          /* webpackInclude: ${webpackIncludeRegexp(specifier)} */
          '${directory}/' + pathRemainder
        );
      }

  `;
}

function toImportFn(stories, {
  needPipelinedImport
} = {}) {
  let pipelinedImport = `const pipeline = (x) => x();`;

  if (needPipelinedImport) {
    pipelinedImport = `
      const importPipeline = ${_importPipeline.importPipeline};
      const pipeline = importPipeline();
    `;
  }

  return (0, _tsDedent.dedent)`
    ${pipelinedImport}

    const importers = [
      ${stories.map(toImportFnPart).join(',\n')}
    ];

    export async function importFn(path) {
      for (let i = 0; i < importers.length; i++) {
        const moduleExports = await pipeline(() => importers[i](path));
        if (moduleExports) {
          return moduleExports;
        }
      }
    }
  `;
}